"""Popup-form-specific HttpResponse classes"""
from django.http import HttpResponseRedirect


class OpenFormResponse(HttpResponseRedirect):
    """Redirects back to the referer, re-opening the popup form"""

    def __init__(self, request, form=None, redirect_to=None):
        if form:
            request.session['popup_form'] = (request.path,
                                             form.data, form.errors)
        else:
            request.session['popup_form'] = request.path, None, None

        if redirect_to is None:
            redirect_to = request.META.get('HTTP_REFERER', '/')

        return super(OpenFormResponse, self).__init__(redirect_to)


class CloseFormResponse(HttpResponseRedirect):
    """Redirects back to the referer, closing the popup form"""

    def __init__(self, request, redirect_to=None):

        # Delete old popup form from session
        if 'popup_form' in request.session:
            del request.session['popup_form']

        if redirect_to is None:
            redirect_to = request.META.get('HTTP_REFERER', '/')
        return super(CloseFormResponse, self).__init__(redirect_to)



"""Decorators for popup form processing views"""

from functools import wraps
from django.http import Http404, HttpResponseRedirect


def handler(func):
    """Decorator for popup form handling view.

    Re-populate submitted popup form with error on referrer's page.

    Popup forms (for sending messages, inviting to events, etc.) are
    originally rendered hidden in the template, from which they should
    be sent. This decorator wraps a view processing submission
    of popup form.

    The popup handling view should return one of following objects::

      * popup_forms.CloseFormResponse
      * popup_forms.OpenFormResponse

    In case of returning `CloseFormResponse` the browser is redirected by
    default to the same page, without populating form. In case of returning
    `OpenFormResponse` browser by default is redirected to the same page,
    and the form is re-populated. This decorator puts the form with
    errors to session, and redirects
    back to original view, from where form was submitted. Then
    popup form is re-populated, showing all errors.

    Both `OpenForm` and `CloseForm` have optional `redirect_to`
    argument, specifying the URL to redirect instead of default one.

    .. IMPORTANT::
        * View should not render anything (i.e. return `HttpResponse`).
        * If form validation failed, view should return
          `OpenFormResponse` passing form as only argument::

              if not form.is_valid():
                  return OpenFormResponse(form)

    """

    @wraps(func)
    def wrapper(request, *args, **kwargs):

        # Delete old popup form from session
        if 'popup_form' in request.session:
            del request.session['popup_form']

        # Process the form and redirect to the next URL
        response = func(request, *args, **kwargs)
        if isinstance(response, HttpResponseRedirect):
            return response

        # The view should NOT populate form itself!
        if request.method == 'POST':
            raise ValueError('View for processing popup form populates it!')
        else:
            raise Http404

    return wrapper


def show_popup_form(action, check_function=None):
    """Explicitly shows popup when rendering template in decorated view.

    Works only in case no popup form is defined to be shown
    (i.e. there is no request.session['popup_form']).

    :action: Action URL for which popup should be made visible

    :check_function: Callable, returning boolean value, to determine
    whether popup should be shown or not. If `None`, the popup
    is shown implicitly. Should have the same signature as
    decorated view function: ``func(request, *args, **kwargs)``

    """
    def make_wrapper(func):
        @wraps(func)
        def wrapper(request, *args, **kwargs):
            if ('popup_form' not in request.session
               and (not check_function
                    or check_function(request, *args, **kwargs))):
                request.session['popup_form'] = (action, None, None)
            return func(request, *args, **kwargs)
        return wrapper
    return make_wrapper


def popup_if_session_var(action, session_key):
    """Shows popup form for specified view, if the key found in session."""
    # See `popup_forms.decorators.show_popup_form` decorator for more info.
    def _check_function(request, *args, **kwargs):
        return (request.user.is_authenticated()
                and session_key in request.GET   # for testing purposes
                or session_key in request.session)
    return show_popup_form(action, _check_function)




from django.utils.importlib import import_module
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


def popup_forms(request):
    """Puts all popup forms to 'popup_forms' context variable."""

    popup_forms = {}
    for popup_form in settings.POPUP_FORMS:
        module_name, sep, form_name = popup_form.rpartition('.')
        try:
            mod = import_module(module_name)
        except ImportError, e:
            raise ImproperlyConfigured('Error importing popup form '
                    'from module {0}: "{1}"'.format(module_name, e))
        try:
            popup_forms[form_name] = getattr(mod, form_name)
        except AttributeError:
            raise ImproperlyConfigured('Module "{0}" does not define'
                    ' a "{1}" form class'.format(module_name, form_name))
    return {'popup_forms': popup_forms}



from setuptools import setup

try:
    long_description = open('README.rst').read()
except IOError:
    long_description = ''

setup(
    name='django-popup-forms',
    version='1.0.1',
    description='Django pop-up forms framework.',
    long_description=long_description,
    author='Social TRM Ltd',
    author_email='david@socialtrm.com',
    url='http://github.com/joinourtalents/django-popup-forms',
    keywords = "django",
    packages=['popup_forms'],
    include_package_data=True,
    zip_safe=False,
    license='BSD License',
    platforms = ['any'],
)



__author__ = 'David Avsajanishvili'
__copyright__ = 'Copyright (c) Social TRM Ltd.'
__license__ = 'BSD'
__all__ = ['OpenFormResponse', 'CloseFormResponse', 'handler']

from popup_forms.responses import CloseFormResponse, OpenFormResponse
from popup_forms.decorators import handler



"""Unittests for Popup Forms functionality"""

from unittest import skip

from django import test, forms
from django.conf.urls.defaults import patterns, url
from django.http import HttpResponse
from django.shortcuts import render

import popup_forms
from django.core.urlresolvers import reverse

try:
    from django.test.utils import override_settings
except ImportError:
    from django.conf import settings

    def override_settings(**kwargs):
        for key, value in kwargs.iteritems():
            setattr(settings, key, value)
        return lambda fn: fn


class PopupForm(forms.Form):
    name = forms.CharField(max_length=10)
    email = forms.EmailField(max_length=20)

    def save(self):
        return '{name}, {email}'.format(**self.cleaned_data)


def index(request):
    return HttpResponse('Hello, World!')


def render_form(request):
    return render(request, 'popup_forms_test/page.html')


@popup_forms.handler
def process_form(request):
    if request.method == 'POST':
        form = PopupForm(request.POST)
        if not form.is_valid():
            return popup_forms.OpenFormResponse(request, form)
        request.session['stored_data'] = form.save()
        return popup_forms.CloseFormResponse(request, reverse('success'))
    return popup_forms.CloseFormResponse(request)


def success(request):
    return HttpResponse(request.session.pop('stored_data', 'No data'))


urlpatterns = patterns('',
    url(r'^$', index, name='index'),
    url(r'^render_form/$', render_form, name='render_form'),
    url(r'^process_form/$', process_form, name='process_form'),
    url(r'^success/$', success, name='success'),
)


@override_settings(POPUP_FORMS=('popup_forms.tests.PopupForm',))
class TestPopupForm(test.TestCase):
    """Unit-testing popup forms"""

    urls = 'popup_forms.tests'

    def test_render_form(self):
        """Form should be rendered by `popup_form` template tag"""
        response = self.client.get('/render_form/')
        self.assertContains(response, '<form method="post" action="/process_form/">')
        self.assertContains(response, '<a href="/process_form/" id="popup_link_1"')
        self.assertContains(response, 'style="display:none"')
        self.assertContains(response, '<input id="id_name" type="text" '
                            'name="name" maxlength="10" />')
        self.assertContains(response, '<input id="id_email" type="text" '
                            'name="email" maxlength="20" />')

    @skip('TODO: Write test!')
    def test_render_form_kwargs(self):
        """Extra Keyword arguments should be passed to form when instantiating it"""
        pass

    def test_submit_form(self):
        """Form processing view should redirect to success page after submit"""
        response = self.client.post('/process_form/',
                    data={'name': 'David', 'email': 'avsd05@gmail.com'},
                    HTTP_REFERER='/render_form/', follow=True)
        self.assertRedirects(response, '/success/')
        self.assertContains(response, 'David, avsd05@gmail.com')

    def test_error_in_form(self):
        """Form should be re-populated on the same page with errors highlighted"""
        response = self.client.post('/process_form/',
                    data={'name': 'David', 'email': 'wrongemail'},
                    HTTP_REFERER='/render_form/', follow=True)
        self.assertRedirects(response, '/render_form/')
        self.assertNotContains(response, 'style="display:none"')
        self.assertContains(response, 'Enter a valid e-mail address.')
        self.assertContains(response, '<input id="id_name" type="text" '
                            'name="name" value="David" maxlength="10" />')
        self.assertContains(response, '<input id="id_email" type="text" '
                            'name="email" value="wrongemail" maxlength="20" />')


@skip('TODO: Write test!')
class TestTokenVarExtractor(test.TestCase):
    """Unittest for TokenVarExtractor """
    pass
